#!/bin/bash

# Ensure that output from apps like keytool are consistent regardless of locale
export LANG=C LC_ALL=C

#Global Variables
_versioninstalling=1.2.1.25
_vcpluginuser=vcplugin
_vcplugingroup=$_vcpluginuser
_installationdirectory=/opt/vcenterplugin
_installerlogdirectory=$_installationdirectory/logs/installer
_configDirectory=$_installationdirectory/config
_vcprops=$_configDirectory/vCenterServerProperties.properties
_backupdirectory=/tmp/vcenterplugin
_defaultPort=18080
_jksFile=StoragevCenterPlugin.jks
_jksFilePath=$_configDirectory/$_jksFile
_thumbprintFile=$_installerlogdirectory/thumbprint.txt
_jksPassword=""
_sha1Key=""
_aliasForJKS=storagearray
_applicationProp="application-prod.properties"
_eulaFile=EULA.txt
_serviceFile=StoragePluginService.service
_serviceFilePath=/etc/systemd/system/$_serviceFile
_logDirectory=$_installationdirectory/logs
_javaDirectory=/usr/bin/java

_isUpgrade=0
_isRepair=0
_isUninstall=0
_isInstall=0
_isUninstallLocally=0
_isSilentProcess=0
certificateReturn=$?

_localip=
regexForIPv4='^[0-9a-fA-F:.]*$'
regexForIPv6='^([0-9a-fA-F]{0,4}:){1,7}[0-9a-fA-F]{0,4}$'

# test if we are running in a container
containerized()
{
        test "$RUNNING_IN_VCP_CONTAINER" || egrep -q '/docker|/lxc|/kube' /proc/self/cgroup
}

# set permissions for installation folder
set_permissions()
{
    containerized || useradd -m $_vcpluginuser >/dev/null 2>&1
    chown -R $_vcpluginuser.$_vcplugingroup $_installationdirectory
}

#
# get_local_ip() ensures that _localip is set to the local source IP
# address appropriate for use in communicating with the vCenter server.
# Accepts the vCenter IP address/hostname as $1, and sets $_localip
# as a side-effect.
#
get_local_ip()
{
    #if localip is not provided by user, default system ip will be used as a server ip where backend service is considered to be running
    if [ -z "$_localip" ]; then
        # Get vCenter IP address, stripping off any []'s in case it's an IPv6 address
        VCENTER_HOST=$(echo "$1" | sed -e 's/^\[\(.*\)\]/\1/')

        # If it's a hostname, translate it to an IP address using DNS or /etc/hosts
        if [[ $VCENTER_HOST =~ $regexForIPv6 ]] || [[ $VCENTER_HOST =~ $regexForIPv4 ]] ; then
            REMOTE_IP=$VCENTER_HOST
        else
            REMOTE_IP=$(getent ahosts $VCENTER_HOST|cut -d' ' -f1|sort -u|head -1)
        fi

        # Find the source address we'll use to talk to it
        _localip=$(ip route get to $REMOTE_IP | grep -v ^unreach | sed -n 's/.* src \([^ ]*\).*/\1/p')
        if [[ $_localip =~ $regexForIPv6 ]]; then
            _localip="["$_localip"]"
        fi
    fi
}

check_vcprops()
{
    if [ ! -f $_vcprops ]; then
        echo "Configuration file $_vcprops is missing. Please uninstall the VCP plugin manually before re-installing."
        exit 1
    fi
}

get_sha1_key()
{
    _sha1Key=$(keytool -list -rfc -storepass ${2:-$_jksPassword} -keystore ${1:-$_jksFilePath} |
                   openssl x509 -noout -fingerprint -sha1 | tr A-Z a-z | sed s/^[^=]*=//)
    test -n "$_sha1Key" || {
        echo >&2 ERROR: failed to obtain SHA1 thumbprint from ${1:-$_jksFilePath}
    }
}

createStorageCertificate()
{
    if [ -f $_jksFilePath ]; then
        rm -rf $_jksFilePath
    fi
    _jksPassword="$(date +%s | sha256sum | base64 | head -c 8)"

    exts=$( (
              # PhotonOS hostname(1) lacks -A option
              for hostname in $(hostname) $(hostname -A 2>/dev/null); do
                  test -n "$hostname" && echo dns:$hostname
              done
              # PhotonOS hostname(1) lacks -I option
              for addr in $(hostname -I 2>/dev/null || ip a show scope global | sed -nre 's/.*inet6? ([0-9a-f:.]*).*/\1/p'); do
                  test -n "$addr" && echo ip:$addr
              done
          ) | sort -u)

    opts=$(echo $(echo $exts) | sed -e 's/ /,/g' -e 's/^/-ext san=/')

    keytool -genkey -alias $_aliasForJKS -keyalg RSA -keysize 2048 -validity 3650 \
            -keystore $_jksFilePath -storetype pkcs12 -storepass $_jksPassword $opts 2>/dev/null <<EOF
$(hostname --fqdn)
n/a
n/a
n/a
n/a
n/a
y
EOF

    get_sha1_key
}

write_service_file()
{
    echo "Creating storage plugin service..."

    sed -i 's|ExecStart=.*|ExecStart='$_javaDirectory' -jar vcplugin-backend-services.jar --server.port='$_portnumber' |' $_serviceFile

    sed -i 's|WorkingDirectory=.*|WorkingDirectory='$_installationdirectory'|' $_serviceFile

    mv $_serviceFile /etc/systemd/system/

    chmod 0755 $_serviceFilePath

    systemctl daemon-reload

    systemctl enable $_serviceFile

    echo "Storage plugin service created."
}

register__plugin()
{
    echo $response_register >$_installerlogdirectory/registerlog.txt
    test=`grep not $_installerlogdirectory/registerlog.txt | wc -l`

    test2=`grep already $_installerlogdirectory/registerlog.txt | wc -l`
    regex='^\[([0-9a-fA-F]{0,4}:){1,7}[0-9a-fA-F]{0,4}\]$'

    if [[ "$test" -ne 0 ]]; then
        _password1=`echo "$_password" | sed -r 's/[+]+/%2B/g'`

        curl -s -g -i -k -H  "Content-Type:application/json" -X GET 'https://localhost:'$_portnumber'/storage/pluginregistration/registerPlugin?vcenterIP='$_vCenterIP'&username='$_username'&password='$_password1'&company=company&version='$_versioninstalling'&serverThumbprint='$_sha1Key'&zipPathURL&ipAddress='$_localip'&portNumber='$_portnumber''> $_installerlogdirectory/curloutputregister.txt

        registrationResponse=`grep "Response" $_installerlogdirectory/curloutputregister.txt`


        if [[ $registrationResponse != *"Plugin registration successful"* ]]; then
            echo $registrationResponse
            echo "Logs are kept at the location "$_parentDirectory/vcpluginlogs
            cleanUpAfterRegistration
        else
            echo $registrationResponse
        fi


        if ! containerized
        then
            firewall-cmd --zone=public --list-ports >$_installerlogdirectory/ports.txt 2> $_installerlogdirectory/firewall-errors.txt
            _portopen=`grep $_portnumber $_installerlogdirectory/ports.txt | wc -l`

            if [[ "$_portopen" -ne 0 ]]; then
                echo
            else
                firewall-cmd --zone=public --add-port=$_portnumber/tcp --permanent > $_installerlogdirectory/firewallportlogs.txt 2>> $_installerlogdirectory/firewall-errors.txt
                firewall-cmd --reload > $_installerlogdirectory/firewallportlogs.txt 2>> $_installerlogdirectory/firewall-errors.txt
            fi

            if test -s $_installerlogdirectory/firewall-errors.txt
            then
                echo
                echo ERROR: Firewall port may not have been opened successfully.
                cat  $_installerlogdirectory/firewall-errors.txt
                echo
                echo If a firewall is enabled on this host, please use the appropriate method
                echo to permanently allow access to port $_portnumber/tcp before continuing.
                echo
            fi >&2
        fi

        echo Installation complete.

    elif [[ "$test2" -ne 0 ]]; then
        exit
    else
        echo $response_register
    fi
}

#Cleaning acitivity in case error has come while installing the plugin

cleanUpAfterRegistration()
{
        #Stop registered service
        systemctl stop $_serviceFile

        #Remove the service file
        rm -rf $_serviceFilePath

        #Copy log folders to another folder for looking out the logs
        _parentDirectory="$(dirname "$_installationdirectory")"

        mkdir -p $_parentDirectory/vcpluginlogs

        cp -rf  $_logDirectory $_parentDirectory/vcpluginlogs

        cleanUp
}



check_response()
{

    flag=`echo "$responsecurl"|awk '{print match($0,"Success")}'`

    if [ $flag -ne 0 ]; then
            _password1=`echo "$_password" | sed -r 's/[+]+/%2B/g'`
            curl -s -g -i -k -H "Content-Type: application/json" -X GET 'https://localhost:'$_portnumber'/storage/pluginregistration/isPluginAlreadyRegistered?vcenterIP='$_vCenterIP'&username='$_username'&password='$_password1''> $_installerlogdirectory/curloutputLogin.txt
            response_register=`grep "Response" $_installerlogdirectory/curloutputLogin.txt`


            echo $response_register

            register__plugin $response_register
    else
            #In case of failure, a new directory will be created where logs will be copied and existing installation directory will be deleted.
            systemctl stop $_serviceFile
            rm -rf $_serviceFilePath

            _parentDirectory="$(dirname "$_installationdirectory")"

            mkdir -p $_parentDirectory/vcpluginlogs

            cp -rf  $_logDirectory $_parentDirectory/vcpluginlogs

            echo "$responsecurl"
            #processid_afterrestarting=`/bin/ps -fu $USER| grep "vcplugin-backend-services" | grep -v "grep" | awk '{print $2}'`

            echo "Logs can be found in "$_parentDirectory/vcpluginlogs
            #kill -9 $processid_afterrestarting

            cleanUp
    exit
    fi
}

isPortValid ()
{
    if [[ "$_portnumber" -gt 1023 &&  "$_portnumber" -lt 49152 ]]; then
        _portvalid="success"
    else
        _portvalid="fail"
    fi
}

isIpValid()
{
    if [ -f "$_vcprops" ]; then
        _verifyip=`grep vCenterServerIp $_vcprops | tail -n1 | awk -F= '{print $2}'`
    fi
    if [ "$_vCenterIP" == "$_verifyip" ] || [[ -z "${_verifyip}" ]]; then

        local IFS=.; local -a a=($_vCenterIP)
        # Start with a regex format test
        [[ $_vCenterIP =~ ^[0-9]+(\.[0-9]+){3}$ ]] || return 1
        # Test values of quads
        for quad in {0..3}; do
            [[ "${a[$quad]}" -gt 255 ]] && return 1
        done
        return 0
    else
        echo "Cannot install plugin on two vCenter servers."
        rm -rf /tmp/vcplugin/javaversion.txt
        exit 1
    fi
}

isPasswordValid()
{
    if [[ ${#_password} -ge 8 && "$_password" == *[a-z]* && "$_password" == *[0-9]* && "$_password" == *['!'@#\$%^\&*()_+]* ]]; then
        _passwordvalid="success"
    else
        _passwordvalid="fail"
    fi
}

isUsernameValid()
{
    if [[ "$_username" == *[a-z]* && "$_username" == *['!'@#\$%^\&*()_+]* ]]; then
        _usernamevalid="success"
    else
        _usernamevalid="fail"
    fi
}

var_execute_setup ()
{
    _ipvalid="success"

    if [ -z "$_vCenterIP" ] || [ -z "$_username" ] || [ -z "$_portnumber" ] ; then
        echo "Credentials cannot be empty. Please provide correct credentials and run again."
        exit 1
    else
        if [[ $_ipvalid == "fail" ]]; then
            echo "IP address is not valid."
            exit 1
        elif [[ $_usernamevalid == "fail" ]]; then
            echo "Username is not valid."
            exit 1
        elif [[ $_portvalid == "fail" ]]; then
            echo "Port number is not valid. Please enter the port number in the range of registered ports i.e. 1024-49151."
            exit 1
        else
            :
        fi
    fi


    isPasswordValid _password
    if [[ -z "${_password}" ]]; then
        echo "Password cannot be empty. Please provide Password and run again."
        exit 1
    else
        if [[ $_passwordvalid == "fail" ]]; then
            echo "Please enter password which is atleast 8 characters long and must contains atleast one uppercase , one lowercase and one special character."
            exit 1
        else
            :
        fi
    fi
}

check_login ()
{
    _password1=`echo "$_password" | sed -r 's/[+]+/%2B/g'`
    responsecurl=$(curl -s -g -i -k -H "Content-Type:application/json" -X GET 'https://localhost:'$_portnumber'/storage/pluginregistration/loginTovCenterServer?vcenterIP='$_vCenterIP'&username='$_username'&password='$_password1''| grep Response)
    check_response $responsecurl
}

function updateZipAndController ()
{
    port=$1
    _jwtpassword="$(date +%s | sha256sum | base64 | head -c 32)"
    #Extract JAR as cotnroller need IP and port no on which backend serivce is running.

    cd $_installationdirectory

    jar -xf vcplugin-backend-services.jar

    cd BOOT-INF/classes/public/

    if [ $_isInstall -eq 1 ]
    then

        cp $_jksFilePath ../

        sed -i 's/server.ssl.key-store-password=.*/server.ssl.key-store-password='$_jksPassword'/' ../application-prod.properties

        sed -i 's/server.ssl.key-store=classpath:.*/server.ssl.key-store=classpath:'$_jksFile'/' ../application-prod.properties

        sed -i 's/server.ssl.key-alias=.*/server.ssl.key-alias='$_aliasForJKS'/' ../application-prod.properties

        sed -i 's|root.path=.*|root.path='$_installationdirectory'|' ../application-prod.properties


    fi

    if [ $_isRepair -eq 0 ]; then
        sed -i 's/spring.user.password=.*/spring.user.password='$_jwtpassword'/' ../application-prod.properties
    fi

    unzip vcplugin-bundle.zip > $_installerlogdirectory/vcplugin-bundle_extraxt.txt

    cd plugins/

    jar -xf vcplugin-controller.jar

    sed -i 's/remote.host.ip=.*/remote.host.ip='$_localip'/' Config.properties
    sed -i 's/remote.host.port=.*/remote.host.port='$1'/' Config.properties
    if [ $_isRepair -eq 0 ]; then
        sed -i 's/rest.user.password=.*/rest.user.password='$_jwtpassword'/' Config.properties
    fi

    jar cfm0 vcplugin-controller.jar META-INF/MANIFEST.MF Config.properties META-INF/ vcplugin/

    rm -rf Config.properties META-INF/ vcplugin/

    cd ..

    zip -r vcplugin-bundle.zip plugin-package.xml plugins/ > $_installerlogdirectory/vcplugin-bundle_zip.txt

    rm -rf plugins/ plugin-package.xml

    cd ../../..

    jar cfm0 vcplugin-backend-services.jar META-INF/MANIFEST.MF BOOT-INF/ org/

    rm -rf META-INF/ BOOT-INF/ org/
}

function is_port_free ()
{

    resp=`lsof -i -P -n | grep ":"$_portnumber`

    if [ -z "$resp" ]; then

        updateZipAndController $_portnumber

        cd $_installationdirectory

        if [ $_isInstall -eq 1 ]
        then
            containerized || write_service_file
        fi

        systemctl start $_serviceFile
    else
        echo $_portnumber is not available. Choose another port and run again.
        exit 1
    fi

    end=$((SECONDS+180))

    _isApplicationRunning="False"
    while [[ SECONDS -lt $end  ]]; do
        curl -s -g -i -k -X GET 'https://localhost:'$_portnumber'/storage/pluginregistration/isApplicationRunning' > $_installerlogdirectory/appisrunning.txt

        response_appisrunning=`grep Running $_installerlogdirectory/appisrunning.txt`

        checkJarStarted=`echo "$response_appisrunning"|awk '{print match($0,"Storage Plugin Is Running")}'`

        if [[ "$checkJarStarted" -ne 0 ]]; then
            _isApplicationRunning="True"
            _portfreeresponse="Success"
            break
        fi
        sleep 5
    done

    if [[ "$_isApplicationRunning" == "False" ]]; then
        echo "Unable to start java application. Please try again."
        exit 1
    fi
}

function check_process_id()
{
    if [[ -z "${process_id}"  ]]; then

            echo -e "\nChecking port may take 1-2 minutes..."
            is_port_free
            if [[ "$_portfreeresponse" == "Success" ]] ; then
                echo "Checking authentication. This may take 1-2 minutes..."
                check_login
        fi

    else

        while [[ -z "$processIdNotSet" ]] ; do

        if [[ -z "$var5" ]] ; then
             echo "Application already running. Do you want to kill and rerun it?(yes/no)"
             read var5
             processIdNotSet=$var5
        fi
        case $processIdNotSet in
                [Yy][eE][sS])
                    echo -e "\nKilling already running application and restarting"
                    #kill -9 $process_id
                    systemctl stop $_serviceFile
                     echo -e "\nChecking port may take 1-2 minutes"
                            is_port_free
                             if [[ "$_portfreeresponse" == "Success" ]] ; then
                                    echo "Checking authentication. This may take 1-2 minutes..."
                        check_login
                    fi
                    ;;
                [Nn][Oo])
                    echo -e "\nSkipping Setup..\n"
                    exit 1
                    ;;
                *)
                    echo "Wrong answer. Print 'yes' or 'no'"
                                               unset processIdNotSet
                    unset var5
                    ;;
                        esac
                     done
    fi
}


initialise_installation()
{
    process_id=`/bin/ps -fu $_vcpluginuser| grep "vcplugin-backend-services" | grep -v "grep" | awk '{print $2}'`

    createStorageCertificate

    _isInstall=1

    check_process_id $process_id
}

#To check whether Java is installed on the machine or not.
check_java ()
{
    # if java and jar are in our $PATH already, that's enough
    (which java && which jar)>/dev/null 2>&1 && return

if [[ -n "$JAVA_HOME" ]] && [[ -x "$JAVA_HOME/jre/bin/java" ]];  then
    _java="$JAVA_HOME/jre/bin/java"

     mkdir /tmp/vcplugin

     $_java -version 2> /tmp/vcplugin/javaversion.txt
     version=`cat /tmp/vcplugin/javaversion.txt | grep "version" | awk -F. {'print $2'}`

    if [ $version -lt 8 ]
    then
            echo "Java version should be 1.8"
            echo "Please install JAVA 1.8 and Start the installation process again"
            echo ''
            rm -rf /tmp/vcplugin/javaversion.txt
            exit 1

    else
            rm -rf /tmp/vcplugin
    fi

    else
        cat <<EOF
To run this vCenter Plugin, you need to have Java installed on the system.
Please install the Java SDK and set the JAVA_HOME environment variable if
the java and jar commands are not installed in the default locations.
EOF
        exit 1
    fi
}

accept_license()
{
    a=1
    while [ $a -lt 10 ] ; do
        (cat <<EOF; tr -d '\r' < $_eulaFile | fmt) | /bin/less -Me -

You must accept the terms of End User License Agreement below before
installing the plugin. Press the space key to proceed to the next screen.

EOF
    echo -e "\n
Do you accept the terms of the End User License Agreement? (yes/no)\n"
    read _varLicense
    case $_varLicense in
        [Yy][eE][sS])
                return
                ;;
            [Nn][Oo]) echo "Exiting Installation."
                exit 1
                return # for testing
                ;;
            *) echo "Please enter 'Yes' or 'No' after the EULA is re-displayed."
                sleep 2
                ;;
    esac
    a=`expr $a + 1`
done

    echo "\nToo many retries.  Exiting installation."
    exit 1
}

retrieve_credentials ()
{
    _vCenterIP=`grep vCenterServerIp $_vcprops | tail -n1 | awk -F= '{print $2}'`
        _vCenterIP="${_vCenterIP//\\:/:}"

    _username=`grep vCenterServerUsername $_vcprops | tail -n1 | awk -F= '{print $2}'`

    _password=`grep vCenterServerPassword $_vcprops | tail -n1 | awk -F= '{print $2}'`
        _password=${_password:0:${#_password}-1}'=='

    _portnumber=`grep 'application.port.number' $_vcprops | tail -n1 | awk -F= '{print $2}'`
}

unregister_Plugin_Completely()
{
    echo "Unregistering the plugin from vCenter" $_vCenterIPUninstall "may take 2-3 minutes."

    systemctl is-active $_serviceFile > $_installerlogdirectory/appisrunningforunregister.txt

    if ! grep -q active "$_installerlogdirectory/appisrunningforunregister.txt"; then
        if [ -d $_installationdirectory ]; then

            echo "Application is not running, restarting the application now..."
            check_vcprops

            _portnumber=`grep 'application.port.number' $_vcprops | tail -n1 | awk -F= '{print $2}'`

            if [ ! -f $_installationdirectory/vcplugin-backend-services.jar ]; then
                is_port_free
            else
                cp -rf vcplugin-backend-services.jar $_installationdirectory/vcplugin-backend-services.jar
                is_port_free
            fi

            if [[ "$_portfreeresponse" == "Success" ]] ; then
                echo "Application started successfully, uninstalling the plugin now."
            fi
        else
            echo "Plugin is not installed."
            exit 1
        fi
    fi

    check_vcprops

    retrieve_credentials
    sleep 10
    curl -s -g -i -k -X GET 'https://localhost:'$_portnumber'/storage/pluginregistration/unRegisterPlugin?vcenterIP='$_vCenterIP'&username='$_username''  -H 'content-type: application/json'  -H 'password: '$_password'' > $_installerlogdirectory/curloutputunregister.txt
    response_unregister=`grep Response $_installerlogdirectory/curloutputunregister.txt`

    flagUnistall=`echo "$response_unregister"|awk '{print match($0,"Success")}'`

    if [ $flagUnistall -ne 0 ]; then
        echo $response_unregister
        uninstall_Plugin_Locally
    else
        echo "Error removing the plugin from the vCenter Server: "$response_unregister
        exit 1
    fi
}

uninstall_Plugin_Locally ()
{
        systemctl stop $_serviceFile
        rm -rf $_installationdirectory 2>&1 | egrep -v 'Device or resource busy'
        rm -rf $_serviceFilePath
}

Unregister_plugin ()
{
if [[ "$_isUninstallLocally" -ne 1 ]]; then
    unregister_Plugin_Completely
else
    uninstall_Plugin_Locally
    echo "Plugin uninstalled locally."
fi
}

uninstall()
{
    check_java
    Unregister_plugin
}

install()
{
    POSITIONAL=()
    while [[ $# -gt 0 ]]
    do
        key="$1"
        case $key in
            -pw|--password)
                _password="$2"
                shift # past argument
                shift # past value
                ;;
            -p|--port)
                _portnumber="$2"
                shift # past argument
                shift # past value
                ;;
            -vc|--vcenter-address)
                _vCenterIP="$2"

                shift # past argument
                shift # past value
                ;;
            -u|--user)
                _username="$2"
                shift # past argument
                shift # past value
                ;;
            -f|--force)
                #No operation for this parameter
                shift # past argument
                ;;
            #localip parameter passed by user if he wants to mention it explicitly(This is the ip of centos/debian system)
            -localip|--local-address)
                _localip="$2"
                regex='^([0-9a-fA-F]{0,4}:){1,7}[0-9a-fA-F]{0,4}$'
                if [[ $_localip =~ $regexForIPv6 ]]; then
                    _localip="["$_localip"]"
                fi
                shift # past argument
                shift # past value
                ;;
            *)
                echo "Invalid arguments specified on the command line: \"$key\""
                plugin_help
                #XXX rm -rf $_installationdirectory
                exit 1
                POSITIONAL+=("$1") # save it in an array for later
                shift # past argument
                ;;
        esac
    done
    set -- "${POSITIONAL[@]}" # restore positional parameters

    if [ -z "$_portnumber" ]; then
        _portnumber=$_defaultPort
    fi

    if [ -z "$_vCenterIP" ] || [ -z "$_username" ] ; then
        plugin_help
        #XXX rm -rf $_installationdirectory
        exit 1
    fi

    get_local_ip $_vCenterIP

    if [[ $_vCenterIP =~ $regexForIPv6 ]]; then
        _vCenterIP="["$_vCenterIP"]"
    fi

    check_prerequisites

    check_java

    echo "The following values will be used to install the plugin:"
    echo "vCenter User Name: "$_username
    echo "vCenter IP Address: "$_vCenterIP
    echo "Local port number: "$_portnumber
    echo "Local IP address: "$_localip

    #If installation is not silent, plugin will ask user to confirm the entered data before proceeding
    if [ $_isSilentProcess -ne 1 ]; then

        a=1
        while [ $a -lt 4 ] ; do
            echo -e "\nAre these correct? (yes/no)"
            read _varCorrectValues
            case $_varCorrectValues in
                [Yy][eE][sS])
                    break
                    ;;
                [Nn][Oo])
                    echo -e "\nExiting Installation..."
                    exit 1
                    ;;
                *) echo "Please enter 'Yes' or 'No'"
                   unset _varCorrectValues
                   ;;
            esac
            a=`expr $a + 1`
        done
    fi

    #License info will not be shown if it is a silent installation
    if [ $_isSilentProcess -ne 1 ]; then
        accept_license
    fi

    if [[ -z $_password ]]; then
        echo "Enter vCenter Server's password"
        read -s _password
    fi

    if [ -d $_installationdirectory ]; then
        #echo "Directory Exists, using the existing directory" $_installationdirectory
        :
    else
        #echo "Creating new directory structure" $_installationdirectory
        mkdir -p $_installationdirectory
    fi

    cp -rf vcplugin-backend-services.jar $_installationdirectory/vcplugin-backend-services.jar

    cp  $_serviceFile $_installationdirectory

    cd $_installationdirectory
    #pwd

    if [ -d "$_installerlogdirectory" ]; then
        #echo "Directory Exists, using the existing directory"
        :
    else
        #echo "Creating new directory structure" $_installerlogdirectory
        mkdir -p $_installerlogdirectory
        #chmod -R a+rwx  $_installerlogdirectory
    fi

    if [ -d "$_configDirectory" ]; then
        #echo "Directory Exists, using the existing directory" $_configDirectory
        :
    else
        #echo "Creating new directory structure" $_configDirectory
        mkdir -p $_configDirectory
    fi

    set_permissions

    isIpValid $_vCenterIP
    isUsernameValid $_username
    isPasswordValid $_password
    isPortValid $_portnumber
    var_execute_setup $_vCenterIP $_username $_password

    echo -e "\nInstallation is in progress..."
    initialise_installation
}

#This method will check the prerequisites are installed all not.
check_prerequisites ()
{
    missing=
    for cmd in openssl keytool lsof unzip zip java jar useradd
    do which $cmd >/dev/null 2>&1 || eval "missing=\"$missing$cmd, \""
    done

    if test -n "${missing}"
    then
        echo >&2 "ERROR: required command(s) are not installed on this system: ${missing%, }"
        echo >&2 "       Please install the appropriate packages and re-run the installer."
        exit 1
    fi
}

plugin_help ()
{
    echo
    echo "OPTIONS"
    echo "-p, --port"
    echo "         Portnumber: Port number should be in the range of the registered ports i.e. between 1024-49151."
    echo "-i, --install"
    echo "         Installation: Starts the installation process of the plugin."
    echo "-un, --uninstall"
    echo "         Uninstallation: Starts the uninstallation process of the plugin."
    echo "-up, --upgrade"
    echo "         Upgrade: Starts the upgradation process of the plugin."
    echo "-r, --repair"
    echo "         Repair: Starts the repair process of the plugin."
    echo "-vc, --vcenter-address"
    echo "         vCenter server IP: vCenter Server IP where plugin is to be registered."
    echo "-u, --user"
    echo "         Username: Username of the vCenter server which should contain atleast one alphabet and one special character."
    echo "-pw, --password"
    echo "         Password: Password of the vCenter server."
    echo "-localip, --local-address"
    echo "         Local IP: IP address of server where backend service will be running."
    echo "-f, --force"
    echo "         Forced operation: Performs the operation without asking for confirmation."

cat <<EOF

EXAMPLES
        bash $0 --install --vcenter-address vcsa.example.com --local-address 10.0.0.123 --user administrator@vsphere.local --password secret-Passw0rd

        bash $0 --upgrade --force

        bash $0 --uninstall
EOF


}


check_version_of_plugin()
{
    check_vcprops

    retrieve_credentials

    if [ -d "$_installerlogdirectory" ]; then
        :
    else
        mkdir -p $_installerlogdirectory
    fi
    set_permissions

    curl -s -g -i -k -H "Content-Type: application/json" -H 'password: '$_password'' -X GET 'https://localhost:'$_portnumber'/storage/pluginregistration/getRegisterPluginVersion?vcenterIP='$_vCenterIP'&username='$_username'' > $_installerlogdirectory/curloutputversion.txt

    _versionintermediate=`grep Success $_installerlogdirectory/curloutputversion.txt`
    _versioninstalling=`echo $_versionintermediate | cut -f3 -d":"`

    if [[ -z "${_versioninstalling// }" ]]; then
        echo "Unable to fetch the plugin version"
        echo "Reinstalling the plugin with same version. It may take 4-5 minutes".

        #               kill -9 $process_id
        systemctl stop $_serviceFile
        updateCertificateForRepair

        is_port_free

        check_login

    else
        echo "Found plugin version" $_versioninstalling
        echo "There is not need for repair"
    fi
    #echo response_getversion is $_versioninstalling
}


updateCertificateForRepair()
{
    _currentDirectory=`pwd`

    cd $_installationdirectory

    if [  -f vcplugin-backend-services.jar ]; then

        jar -xf vcplugin-backend-services.jar

        if [ -d "/tmp/vcenterplugin/certificate/" ]; then
            :
        else
            #echo "Creating new directory structure"
            mkdir -p /tmp/vcenterplugin/certificate

        fi

        if [ -f $_installationdirectory/BOOT-INF/classes/$_jksFile ]; then

            _jksPassword=`grep server.ssl.key-store-password= $_installationdirectory/BOOT-INF/classes/application-prod.properties | awk -F= '{print $2}'`

            get_sha1_key

        else
            echo "Self signed certificate is not created."
            createStorageCertificate

        fi

        jar cfm0 vcplugin-backend-services.jar META-INF/MANIFEST.MF BOOT-INF/ org/

        rm -rf META-INF/ BOOT-INF/ org/

    fi

    cd $_currentDirectory
}

updateCertificationAndConfigurtion()
{
    _isNewCertificate=0
    _currentDirectory=`pwd`

    cd $_installationdirectory

    if [  -f vcplugin-backend-services.jar ]; then

        jar -xf vcplugin-backend-services.jar

        if [ -d "/tmp/vcenterplugin/certificate/" ]; then
            :
        else
            #echo "Creating new directory structure"
            mkdir -p /tmp/vcenterplugin/certificate

        fi

        if [ -f $_installationdirectory/BOOT-INF/classes/$_jksFile ]; then

            cp -rf $_installationdirectory/BOOT-INF/classes/application-prod.properties /tmp/vcenterplugin/certificate/

            cp -rf $_installationdirectory/BOOT-INF/classes/$_jksFile /tmp/vcenterplugin/certificate/

        else
            echo "Self signed certificate is not created."
            createStorageCertificate
            _isNewCertificate=1
        fi

        jar cfm0 vcplugin-backend-services.jar META-INF/MANIFEST.MF BOOT-INF/ org/

        rm -rf META-INF/ BOOT-INF/ org/

    else

        echo "Self signed certificate is not created."
        createStorageCertificate

        _isNewCertificate=1
    fi

    cd $_currentDirectory

    jar -xf vcplugin-backend-services.jar

    if [ $_isNewCertificate -ne 1 ]; then

        cp -rf /tmp/vcenterplugin/certificate/$_jksFile BOOT-INF/classes/

        _jksPassword=`grep 'server.ssl.key-store-password=' /tmp/vcenterplugin/certificate/application-prod.properties | awk -F= '{print $2}'`

        _aliasForJKS=`grep 'server.ssl.key-alias=' /tmp/vcenterplugin/certificate/application-prod.properties | awk -F= '{print $2}'`

        _jksFile=`grep 'server.ssl.key-store=classpath:' /tmp/vcenterplugin/certificate/application-prod.properties | awk -F: '{print $2}'`

        get_sha1_key /tmp/vcenterplugin/certificate/$_jksFile

    else
        cp $_jksFilePath BOOT-INF/classes/
    fi

    sed -i 's/server.ssl.key-store-password=.*/server.ssl.key-store-password='$_jksPassword'/' BOOT-INF/classes/application-prod.properties

    sed -i 's/server.ssl.key-store=classpath:.*/server.ssl.key-store=classpath:'$_jksFile'/' BOOT-INF/classes/application-prod.properties

    sed -i 's/server.ssl.key-alias=.*/server.ssl.key-alias='$_aliasForJKS'/' BOOT-INF/classes/application-prod.properties

    jar cfm0 vcplugin-backend-services.jar META-INF/MANIFEST.MF BOOT-INF/ org/

    rm -rf META-INF/ BOOT-INF/ org/
}

upgrade_plugin()
{
    check_vcprops

    retrieve_credentials
    sleep 10
    if [ -d "/tmp/vcenterplugin" ]; then
        #echo "Directory Exists, using the existing directory"
        :
    else
        #echo "Creating new directory structure"
        mkdir -p /tmp/vcenterplugin
        chmod -R 777 /tmp/vcenterplugin
    fi

    if [ -d $_installationdirectory/db ] && [ -d $_configDirectory ] && [ -d $_installationdirectory/logs ]; then
        cp -rf $_installationdirectory/db /tmp/vcenterplugin/db
        cp -rf $_configDirectory /tmp/vcenterplugin/config
        cp -rf $_installationdirectory/logs /tmp/vcenterplugin/logs
    else
        :
        #Database, log files are missing.
    fi

    systemctl stop $_serviceFile

    #This code will add certificate to the Backend JAR to support HTTPs.
    updateCertificationAndConfigurtion

    rm -rf $_installationdirectory/vcplugin-backend-services.jar
    #echo "copying the new jar of the updated version"
    cp -rf vcplugin-backend-services.jar $_installationdirectory/vcplugin-backend-services.jar

    get_local_ip $_vCenterIP

    set_permissions

    is_port_free
    if [[ "$_portfreeresponse" == "Success" ]] ; then
        #echo "New jar started successfully!!"
        :
    fi

    curl -s -g -i -k -X GET 'https://localhost:'$_portnumber'/storage/pluginregistration/updatePlugin?vcenterIP='$_vCenterIP'&username='$_username'&serverThumbprint='$_sha1Key'&ipAddress='$_localip'&portNumber='$_portnumber''  -H 'content-type: application/json'  -H 'password: '$_password'' > $_installerlogdirectory/curloutputupgrade.txt
    response_upgrade=`grep "Response" $_installerlogdirectory/curloutputupgrade.txt`

    echo $response_upgrade
}

repair_plugin ()
{
    echo $process_id
    if [[ ! -z "${process_id}" ]]; then
        #kill -9 $process_id
        systemctl stop $_serviceFile
    fi

    #echo "Removing the corrupted jar"

    if [ -d "$_backupdirectory" ]; then
        #echo "Directory Exists, using the existing directory"
        :
    else
        #echo "Creating new directory structure"
        mkdir -p $_backupdirectory
        chmod -R a+rw  $_backupdirectory
    fi
    set_permissions

    #This code will add certificate to the Backend JAR to support HTTPs.

    updateCertificationAndConfigurtion

    cp -rf $_installationdirectory/vcplugin-backend-services.jar $_backupdirectory/vcplugin-backend-services.jar
    rm -rf $_installationdirectory/vcplugin-backend-services.jar
    #echo "copying the new jar of the same version"
    cp -rf vcplugin-backend-services.jar $_installationdirectory/vcplugin-backend-services.jar
    is_port_free

    #check_version_of_plugin

    if [[ "$_portfreeresponse" == "Success" ]] ; then
        echo "Repair has been performed successfully."
        exit
    fi
}

repair ()
{
    check_java

    process_id=`/bin/ps -fu $_vcpluginuser| grep "vcplugin-backend-services" | grep -v "grep" | awk '{print $2}'`
    if [ -d $_installationdirectory ]; then

        check_vcprops
        _portnumber=`grep 'application.port.number' $_vcprops | tail -n1 | awk -F= '{print $2}'`

        _versioninstalled=`grep plugin.version $_vcprops | tail -n1 | awk -F= '{print $2}'`

        if [ "$_versioninstalled" == "$_versioninstalling" ]; then
            if [[ ! -z "${process_id}" ]]; then
                check_version_of_plugin
                exit
            fi
            echo "Backend service is not running. Starting the service."
            systemctl start $_serviceFile
            echo "Backend srvice is up and running. Repair is successful"
        elif [ "$_versioninstalling" '>' "$_versioninstalled" ]; then
            echo "You are trying to repair with a newer version. Please use the upgrade command instead."
            exit 1
        else
            echo "You are trying to repair with an older version; this is not supported."
            exit 1
        fi
    else
        echo "Plugin is not installed."
        exit 1
    fi
}


cleanUp ()
{
    if [ -d $_installationdirectory ];then
        rm -rf $_installationdirectory 2>&1 | egrep -v 'Device or resource busy'
    fi
    exit
}


check_for_installed_version()
{
    _oldInstallationDirectory=`grep WorkingDirectory /etc/systemd/system/$_serviceFile | tail -n1 | awk -F= '{print $2}'`

    if [ -f $_oldInstallationDirectory/config/vCenterServerProperties.properties ]; then
        _previousInstalledVersion=`grep plugin.version $_configDirectory/vCenterServerProperties.properties | tail -n1 | awk -F= '{print $2}'`

        if [ "$_previousInstalledVersion" == "$_versioninstalling" ]; then
            echo "You are trying to install plugin with same version. Please use repair command instead"
            plugin_help
            exit
        elif [ "$_previousInstalledVersion" '<' "$_versioninstalling" ]; then
            echo "You are trying to install plugin with upgrade version. Please use upgrade command for upgrading the version."
            plugin_help
            exit
        else
            echo "You are trying to install the plugin with downgrade version and it is not supported. Please do reinstall for downgrade version."
            plugin_help
            exit
        fi
    else
        echo "Another version is already installed. Please uninstall it and then continue."
        exit

    fi
}

upgrade ()
{
    check_java

    process_id=`/bin/ps -fu $_vcpluginuser| grep "vcplugin-backend-services" | grep -v "grep" | awk '{print $2}'`

    if [ -d $_installationdirectory ]; then

        check_vcprops

        _portnumber=`grep 'application.port.number' $_vcprops | tail -n1 | awk -F= '{print $2}'`

        echo "Upgrading the plugin may take 4-5 minutes."

        _versioninstalled=`grep plugin.version $_vcprops | tail -n1 | awk -F= '{print $2}'`

        if [ "$_versioninstalling" '>' "$_versioninstalled" ] || ((isforceProcessParameterFound>0)); then
            upgrade_plugin
        else
            echo "Your plugin is already at version $_versioninstalled, so no upgrade is necessary."
            exit
        fi
    else
        echo "Plugin is not installed."
        exit
    fi

    exit
}

#Starting point of plugin installation

user=`whoami`

if [ "$user" != "root" ]; then
    echo "Root privileges are required to install the plugin."
    exit 1
fi

#to disable ctrl+c
trap '' 2
echo ''
echo '########################################'
echo '  vSphere Client Plugin'
echo '########################################'
echo ''

if [ -z $1 ]; then
    plugin_help
    exit 1
else
    case $1 in
        -h|help)
            _action="help"
            plugin_help
            ;;
        -i|--install|install)
            if [ $# -ge 13 ]
            then
                echo "Please enter valid parameters."
                plugin_help
                exit 1
            else
                if [ -f $_serviceFilePath ]; then
                    containerized || check_for_installed_version
                fi
                _installCommandParameters=""
                isforceProcessParameterFound=0
                isPasswordFound=0
                #Iterate through all the passed parameters to check is it is a silent installation and if password is passed in command line arg
                while [ "$1" != "" ]; do
                    case "$1" in
                        -i|--install|install)
                            shift
                            continue
                            ;;
                    esac
                    _installCommandParameters=$_installCommandParameters"$1 "
                    case $1 in -f|--force)
                                   isforceProcessParameterFound=1
                                   _isSilentProcess=1
                                   ;;
                                -pw|--password)
                                    isPasswordFound=1
                                    ;;
                    esac
                    shift
                done
                #If -f option is passed(i.e. silent installation), then -pw parameter is mandatory to be passed in command line
                if [[ $isforceProcessParameterFound -eq 1 && $isPasswordFound -eq 0 ]]; then
                    echo "Please provide password of vCenter server for silent installation."
                    plugin_help
                    exit 1
                fi
                _action="Install"
                install ${_installCommandParameters}
            fi
            ;;

        -up|--upgrade|upgrade)
            if [ $# == 2 ]; then
                isforceProcessParameterFound=0
                while [ "$1" != "" ]; do
                    case $1 in -f|--force)
                                   isforceProcessParameterFound=1
                                   break
                                   ;;
                    esac
                    shift
                done

                if [[ $isforceProcessParameterFound -ne 1 ]]; then
                    echo "Please enter valid parameters."
                    plugin_help
                    exit 1
                else
                    upgrade
                fi
            elif [ $# == 1 ]; then
                a=1
                while [ $a -lt 4 ] ; do
                    echo -e "\nDo you want to Upgrade the plugin? (yes/no)\n"
                    read _varUpgrade
                    case $_varUpgrade in
                        [Yy][eE][sS]) #echo -e "\nUpgrade of plugin is in process"
                            _action="Upgrade"
                            upgrade
                            ;;
                        [Nn][Oo]) echo -e "\nExiting Upgrade.\n"
                                  exit 1
                                  ;;
                        *) echo "Please enter 'Yes' or 'No'"
                           unset _varUpgrade
                           ;;
                    esac
                    a=`expr $a + 1`
                done
            else
                echo "Please enter valid parameters."
                plugin_help
                exit 1
            fi
            ;;
        -un|uninstall|--uninstall)
            if [ $# == 2 ]; then
                isforceProcessParameterFound=0
                while [ "$1" != "" ]; do
                    case $1 in -f|--force)
                                   isforceProcessParameterFound=1
                                   break
                                   ;;
                    esac
                    shift
                done

                if [[ $isforceProcessParameterFound -ne 1 ]]; then
                    echo "Please enter valid parameters."
                    plugin_help
                    exit 1
                else
                    uninstall
                fi

            elif [ $# == 1 ]; then

                a=1
                while [ $a -lt 4 ] ; do
                    echo -e "\nDo you want to uninstall the plugin? (yes/no)\n"
                    read _varUninstall

                    case $_varUninstall in
                        [Yy][eE][sS]) echo -e "\nUninstallation is in process"
                                      _action="Uninstall"

                                      uninstall
                                      break
                                      ;;
                        [Nn][Oo]) echo -e "\nSkipping Uninstall.\n"
                                  exit 1
                                  ;;
                        *) echo "Wrong answer. Print 'yes' or 'no'"
                           unset _varUninstall
                           ;;
                    esac
                    a=`expr $a + 1`
                done
            else
                echo "Please enter valid parameters."
                plugin_help
                exit 1
            fi
            ;;

        -r|--repair|repair)
            if [ $# == 2 ]; then
                isforceProcessParameterFound=0
                while [ "$1" != "" ]; do
                    case $1 in -f|--force)
                                   isforceProcessParameterFound=1
                                   break
                                   ;;
                    esac
                    shift
                done

                if [[ $isforceProcessParameterFound -ne 1 ]]; then
                    echo "Please enter valid parameters."
                    plugin_help
                    exit 1
                else
                    _isRepair=1
                    repair
                fi

            elif [ $# == 1 ]; then
                a=1
                while [ $a -lt 4 ] ; do
                    echo -e "\nDo you want to repair the plugin? (yes/no)\n"
                    read _varRepair

                    case $_varRepair in
                        [Yy][eE][sS])
                            echo -e "\nRepair is in process. This may takes 4-5 minutes.\n"
                            _action="Repair"
                            _isRepair=1
                            repair
                            break
                            ;;
                        [Nn][Oo])
                            echo -e "\nSkipping Repair.\n"
                            exit 1
                            ;;
                        *)
                            echo "Wrong answer. Print 'yes' or 'no'"
                            unset _varRepair
                            ;;
                    esac
                    a=`expr $a + 1`
                done
            else
                echo "Please enter valid parameters."
                plugin_help
                exit 1
            fi
            ;;

        -unl|--uninstall-local*|uninstall-local*)
            if [ $# == 2 ]; then
                isforceProcessParameterFound=0
                while [ "$1" != "" ]; do
                    case $1 in -f|--force)
                                   isforceProcessParameterFound=1
                                   break
                                   ;;
                    esac
                    shift
                done

                if [[ $isforceProcessParameterFound -ne 1 ]]; then
                    echo "Please enter valid parameters."
                    echo "-unl, --uninstall-locally"
                    echo "          Uninstall Locally: Starts the local uninstallation process of the plugin."
                    echo "-f, --force"
                    echo "          Forced Operation: Performs the operation without asking for confirmation."
                    exit 1
                else
                    _isUninstallLocally=1
                    uninstall
                fi
            elif [ $# == 1 ]; then
                a=1
                while [ $a -lt 4 ] ; do
                    echo -e "\nDo you want to uninstall the plugin locally? (yes/no)\n"
                    read _varUninstallLocally

                    case $_varUninstallLocally in
                        [Yy][eE][sS])
                            echo -e "\nUninstallation is in process"
                            _action="Uninstall"
                            _isUninstallLocally=1
                            uninstall
                            break
                            ;;
                        [Nn][Oo])
                            echo -e "\nSkipping Uninstall.\n"
                            exit 1
                            ;;
                        *)
                            echo "Wrong answer. Print 'yes' or 'no'"
                            unset _varUninstallLocally
                            ;;
                    esac
                    a=`expr $a + 1`
                done
            else
                echo "Please enter valid parameters."
                echo "-unl, --uninstall-locally"
                echo "          Uninstall Locally: Starts the local uninstallation process of the plugin."
                echo "-f, --force"
                echo "          Forced Operation: Performs the operation without asking for confirmation."
                exit 1
            fi
            ;;
        *)
            echo "Invalid arguments specified."
            plugin_help
            exit 1
            ;;
    esac
fi
